use std::fmt;
use std::fmt::Display;


#[derive(Clone, Copy, Eq, Debug)]
pub enum LogLevel
{
   /// Nothing will be written to the logs.
   OFF,

   /// Only errors will be written to the logs.
   ///
   /// This designates a serious, program
   /// stopping, error.
   ERROR,

   /// Warnings and errors will be written to
   /// the logs.
   ///
   /// Designates a problem, but it is not
   /// something that will stop the program
   /// from running.
   WARN,

   /// Messages, warnings, and errors will be
   /// written to the logs.
   ///
   /// Designates useful information 
   MESSAGE,

   /// Debug statements, messages, warnings,
   /// and errors will be written to the logs.
   DEBUG,

   /// 
   TRACE
}

impl LogLevel
{
   pub fn from_u8(lvl: u8) -> Option<LogLevel>
   {
      match lvl
      {
         0 => Some(LogLevel::OFF),
         1 => Some(LogLevel::ERROR),
         2 => Some(LogLevel::WARN),
         3 => Some(LogLevel::MESSAGE),
         4 => Some(LogLevel::DEBUG),
         5 => Some(LogLevel::TRACE),
         _ => None
      }
   }
}

impl PartialEq for LogLevel
{
   fn eq(&self, other: &LogLevel) -> bool
   {
      *self as usize == *other as usize
   }
}

impl Display for LogLevel
{
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result
   {
      match *self
      {
         LogLevel::OFF => write!(f, "Off"),
         LogLevel::ERROR => write!(f, "Error"),
         LogLevel::WARN => write!(f, "Warn"),
         LogLevel::MESSAGE => write!(f, "Message"),
         LogLevel::DEBUG => write!(f, "Debug"),
         LogLevel::TRACE => write!(f, "Trace")
      }
   }
}
